@@ -2,7 +2,7 @@
 
 import asyncio
 import json
-from typing import Any, Callable, Dict, Union
+from typing import Any, Callable, Dict, Union, Optional
 from dataclasses import asdict, dataclass
 from enum import Enum
 import uuid
@@ -11,6 +11,7 @@ import logging
 from mcp.server.fastmcp.tools import Tool as MCPToolClass
 
 from notebook_intelligence.config import NBIConfig
+from notebook_intelligence.ruleset import RuleContext
 
 log = logging.getLogger(__name__)
 
@@ -29,6 +30,7 @@ class BackendMessageType(str, Enum):
     StreamEnd = 'stream-end'
     RunUICommand = 'run-ui-command'
     GitHubCopilotLoginStatusChange = 'github-copilot-login-status-change'
+    MCPServerStatusChange = 'mcp-server-status-change'
 
 class ResponseStreamDataType(str, Enum):
     LLMRaw = 'llm-raw'
@@ -49,6 +51,17 @@ class BuiltinToolset(str, Enum):
     NotebookExecute = 'nbi-notebook-execute'
     PythonFileEdit = 'nbi-python-file-edit'
 
+class MCPServerStatus(str, Enum):
+    NotConnected = 'not-connected'
+    Connecting = 'connecting'
+    Disconnecting = 'disconnecting'
+    FailedToConnect = 'failed-to-connect'
+    Connected = 'connected'
+    UpdatingToolList = 'updating-tool-list'
+    UpdatedToolList = 'updated-tool-list'
+    UpdatingPromptList = 'updating-prompt-list'
+    UpdatedPromptList = 'updated-prompt-list'
+
 class Signal:
     def __init__(self):
         self._listeners = []
@@ -95,6 +108,8 @@ class ChatRequest:
     prompt: str = ''
     chat_history: list[dict] = None
     cancel_token: CancelToken = None
+    # NEW: Add context for rule evaluation
+    rule_context: Optional[RuleContext] = None
 
 @dataclass
 class ResponseStreamData:
@@ -372,18 +387,55 @@ class SimpleTool(Tool):
             fn_args.update({"request": request, "response": response})
         return await self._tool_function(**fn_args)
 
+class PromptArgument:
+    @property
+    def name(self) -> str:
+        raise NotImplemented
+    
+    @property
+    def description(self) -> str:
+        raise NotImplemented
+    
+    @property
+    def required(self) -> bool:
+        raise NotImplemented
+
+class MCPPrompt:
+    @property
+    def name(self) -> str:
+        raise NotImplemented
+
+    @property
+    def title(self) -> str:
+        raise NotImplemented
+    
+    @property
+    def description(self) -> str:
+        raise NotImplemented
+
+    @property
+    def arguments(self) -> list[PromptArgument]:
+        raise NotImplemented
+
+    def get_value(self, prompt_args: dict = {}) -> str:
+        raise NotImplemented
+
 class MCPServer:
     @property
     def name(self) -> str:
         return NotImplemented
+
+    @property
+    def status(self) -> MCPServerStatus:
+        return NotImplemented
     
-    async def connect(self):
+    def connect(self):
         return NotImplemented
 
-    async def disconnect(self):
+    def disconnect(self):
         return NotImplemented
 
-    async def update_tool_list(self):
+    def update_tool_list(self):
         return NotImplemented
     
     def get_tools(self) -> list[Tool]:
@@ -392,7 +444,19 @@ class MCPServer:
     def get_tool(self, tool_name: str) -> Tool:
         return NotImplemented
 
-    async def call_tool(self, tool_name: str, tool_args: dict):
+    def call_tool(self, tool_name: str, tool_args: dict):
+        return NotImplemented
+
+    def update_prompts_list(self):
+        return NotImplemented
+
+    def get_prompts(self) -> list[MCPPrompt]:
+        return NotImplemented
+
+    def get_prompt(self, prompt_name: str) -> MCPPrompt:
+        return NotImplemented
+    
+    def get_prompt_value(self, prompt_name: str, prompt_args: dict = {}) -> list[dict]:
         return NotImplemented
 
 def auto_approve(tool: SimpleTool):
@@ -776,11 +840,27 @@ class Host:
     def get_mcp_server_tool(self, server_name: str, tool_name: str) -> Tool:
         return NotImplemented
 
+    def get_mcp_server_prompt(self, server_name: str, prompt_name: str) -> MCPPrompt:
+        mcp_server = self.get_mcp_server(server_name)
+        if mcp_server is not None:
+            return mcp_server.get_prompt(prompt_name)
+        return None
+
+    def get_mcp_server_prompt_value(self, server_name: str, prompt_name: str, prompt_args: dict = {}) -> list[dict]:
+        mcp_server = self.get_mcp_server(server_name)
+        if mcp_server is not None:
+            return mcp_server.get_prompt_value(prompt_name, prompt_args)
+        return None
+
     def get_extension_toolset(self, extension_id: str, toolset_id: str) -> Toolset:
         return NotImplemented
 
     def get_extension_tool(self, extension_id: str, toolset_id: str, tool_name: str) -> Tool:
         return NotImplemented
+    
+    def get_rule_manager(self):
+        """Get the rule manager instance if available."""
+        return NotImplemented
 
 class NotebookIntelligenceExtension:
     @property
